
import itertools
from urllib.parse import urlparse
from lxml import etree
from odoo import http
from odoo.tests import HttpCase, tagged, new_test_user
from odoo.addons.website.tools import MockRequest
from odoo.addons.odoo_challenge.controllers.challenge import Challenge

@tagged('post_install', '-at_install')
class TestChallenge(HttpCase):

    def assertToNextChallengeLevel(self, resp):
        """Assert that request redirect to next challenge level"""
        self.assertEqual(resp.status_code, 303)
        self.assertEqual(resp.headers["Location"], self.base_url() + "/jobs/challenge/next")

    def test_challenge(self):
        ctrl = Challenge()
        self.authenticate(False, False)

        resp = self.url_open('/jobs/challenge')
        resp_html = etree.fromstring(resp.content, parser=etree.HTMLParser())
        csrf_token = resp_html.xpath("//input[@name='csrf_token']")[0].attrib["value"]

        # login
        challenge_user_datas = {'login': 'applicant@example.org', 'csrf_token': csrf_token}
        resp = self.url_open('/jobs/challenge/login', allow_redirects=False, data=challenge_user_datas)
        self.assertToNextChallengeLevel(resp)
        session = http.root.session_store.get(self.session.sid)
        challenge_user = self.env['challenge.user'].sudo().search([
            ('login', '=', challenge_user_datas['login']),
        ])
        self.assertEqual(session.challenge_uid, challenge_user.id)

        # Get and submit response for each challenge level
        for challenge_level in itertools.count(1):
            resp = self.url_open('/jobs/challenge/next')
            resp_css = self.url_open('/jobs/challenge/challenge.css')
            resp_html = etree.fromstring(resp.content, parser=etree.HTMLParser())
            if resp_html.xpath("//h1[contains(., 'Congratulations applicant@example.org')]"):
                # challenge finished
                break

            session = http.root.session_store.get(self.session.sid)  # reload session
            self.assertEqual(
                session.challenge_current_level, challenge_level,
                "Session challenge level should match the expected challenge level"
            )
            with MockRequest(self.env) as mock_request:
                mock_request.session = session
                level = ctrl._get_current_level()
                pwd = ctrl._challenge_pwd()
            self._logger.info("test_challenge - level %s (%s)", challenge_level, level.tech_name)

            if level.tech_name == 'json_prime':
                # ensure user are able to load the JSON
                json_prime_url = resp.headers['It-Is-The-Part-1-Of-Url']
                resp_prime = self.url_open(json_prime_url, allow_redirects=False)
                self.assertEqual(resp_prime.status_code, 303, "Bad url, should redirect to main page")
                self.assertEqual(resp_prime.headers['Location'], self.base_url() + '/jobs/challenge?retry')
                json_prime_url = json_prime_url.replace(
                    '<part2>', urlparse(resp_css.cookies['It-Is-The-Part-2-Of-Url']).path.split('/')[4],
                )
                resp_prime = self.url_open(json_prime_url, allow_redirects=False)
                self.assertEqual(resp_prime.status_code, 200)
                self.assertEqual(
                    set(resp_prime.json().keys()),
                    {'instructions', 'numbers'},
                    'Json Prime should contain both numbers and instructions',
                )
            elif level.tech_name == 'img_iframe':
                # check loading of the iframe
                resp_iframe = self.url_open('/jobs/challenge/challenge')
                self.assertEqual(resp_iframe.status_code, 200)
                self.assertTrue(pwd, resp_iframe.content.decode())
            elif level.tech_name == '57_75':
                resp_check = self.url_open('/jobs/challenge/70/wrong-soluce')
                self.assertEqual(resp_check.status_code, 418)

            # submit response
            challenge_vals = {'pwd': pwd, 'csrf_token': challenge_user_datas['csrf_token']}
            resp = self.url_open("/jobs/challenge/submit", allow_redirects=False, data=challenge_vals)
            self.assertToNextChallengeLevel(resp)

        session = http.root.session_store.get(self.session.sid)  # reload session
        self.assertFalse(session.challenge_current_level)
        self.assertEqual(session.challenge_uid, challenge_user.id)

        resp = self.url_open("/jobs/challenge/logout")
        self.assertEqual(resp.status_code, 200)
        session = http.root.session_store.get(self.session.sid)  # reload session
        self.assertFalse(session.challenge_uid)

    def test_hr_access(self):
        if not self.env["ir.module.module"].search([('name', '=', 'hr'), ('state', '=', 'installed')]):
            self.skipTest("skipping test, needing 'hr' module")

        hr_user = new_test_user(
            self.env,
            login='hr@mycompany.example.net',
            groups="base.group_user,hr.group_hr_user"
        )
        self.authenticate(hr_user.login, hr_user.login)
        challenge_user_datas = {'login': 'applicant@example.org', 'csrf_token': http.Request.csrf_token(self)}
        resp = self.url_open('/jobs/challenge/login', allow_redirects=False, data=challenge_user_datas)
        self.assertToNextChallengeLevel(resp)

        resp = self.url_open('/jobs/challenge/soluces')
        self.assertEqual(resp.status_code, 200)
        self.assertEqual(len(resp.content.decode().split('\n')), 12)
